unit JclWin32Ex;

{$I jcl.inc}

interface

uses
  Windows;

  function JclTryEnterCriticalSection(lpCriticalSection: TRTLCriticalSection):  Boolean;
  function JclSignalObjectAndWait(hObjectToSignal: THandle; hObjectToWaitOn: THandle; dwMilliseconds: Cardinal; bAlertable: Boolean): Cardinal;
  function JclSetCriticalSectionSpinCount(lpCriticalSection: TRTLCriticalSection; dwSpinCount: Cardinal): Cardinal;
  function JclOpenWaitableTimer(dwDesiredAccess: Cardinal; bInheritHandle: Boolean; const lpTimerName: string): THandle;
  function JclInitializeCriticalSectionAndSpinCount(lpCriticalSection: TRTLCriticalSection; dwSpinCount: Cardinal): Boolean;
  function JclGetFileAttributesEx(const lpFileName: string; fInfoLevelId: TGetFileExInfoLevels; lpFileInformation: Pointer): Boolean;
  function JclCreateWaitableTimer(lpTimerAttributes: PSecurityAttributes; bManualReset: Boolean; const lpTimerName: string): THandle;
  function JclCancelWaitableTimer(hTimer: THandle): Boolean;

  procedure CheckAndInitializeOpenGL;

  function JclglGetString(name: Cardinal): PChar;
  function JclglGetError: Cardinal;

  function JclwglCreateContext(hdc: HDC): HGLRC;
  function JclwglDeleteContext(hglrc: HGLRC): BOOL;
  function JclwglMakeCurrent(hdc: HDC; hglrc: HGLRC): BOOL;

  function JclgluErrorString(errCode: Cardinal): PChar;



implementation

type
  TTryEnterCriticalSectionFun = function(lpCriticalSection: TRTLCriticalSection): BOOL; stdcall;
  TSignalObjectAndWaitFun = function(hObjectToSignal: THandle; hObjectToWaitOn: THandle; dwMilliseconds: DWORD; bAlertable: BOOL): DWORD; stdcall;
  TSetCriticalSectionSpinCountFun = function(lpCriticalSection: TRTLCriticalSection; dwSpincCount: DWORD): DWORD; stdcall;
  TInitializeCriticalSectionAndSpinCountFun = function(lpCriticalSection: TRTLCriticalSection; dwSpinCount: DWORD): BOOL; stdcall;

  TOpenWaitableTimerAFun = function(dwDesiredAccess: DWORD; bInheritHandle: BOOL; lpTimerName: LPCTSTR): THandle; stdcall;
  TCreateWaitableTimerAFun = function(lpTimerAttributes: PSecurityAttributes; bManualReset: BOOL; lpTimerName: PAnsiChar): THandle; stdcall;
  TCancelWaitableTimerFun = function(hTimer: THandle): BOOL; stdcall;

  TGetFileAttributesExAFun = function(lpFileName: PChar; fInfoLevelId: TGetFileExInfoLevels; lpFileInformation: Pointer): BOOL; stdcall;

  TglGetStringFun = function(name: Cardinal): PChar; stdcall;
  TglGetErrorFun = function: Cardinal; stdcall;

  TwglCreateContextFun = function (hdc: HDC): HGLRC; stdcall;
  TwglDeleteContextFun = function (hglrc: HGLRC): BOOL; stdcall;
  TwglMakeCurrentFun = function (hdc: HDC; hglrc: HGLRC): BOOL; stdcall;

  TgluErrorStringFun = function(errCode: Cardinal): PChar; stdcall;


var
  Kernel32DllHandle: HMODULE;
  OpenGl32DllHandle: HMODULE;
  Glu32DllHandle: HMODULE;
  OpenGLInitialized: Boolean;
  TryEnterCriticalSectionFun: TTryEnterCriticalSectionFun;
  SignalObjectAndWaitFun: TSignalObjectAndWaitFun;
  SetCriticalSectionSpinCountFun: TSetCriticalSectionSpinCountFun;
  OpenWaitableTimerAFun: TOpenWaitableTimerAFun;
  InitializeCriticalSectionAndSpinCountFun: TInitializeCriticalSectionAndSpinCountFun;
  GetFileAttributesExAFun: TGetFileAttributesExAFun;
  CreateWaitableTimerAFun: TCreateWaitableTimerAFun;
  CancelWaitableTimerFun: TCancelWaitableTimerFun;
  glGetStringFun: TglGetStringFun;
  glGetErrorFun: TglGetErrorFun;
  gluErrorStringFun: TgluErrorStringFun;
  wglCreateContextFun: TwglCreateContextFun;
  wglDeleteContextFun: TwglDeleteContextFun;
  wglMakeCurrentFun: TwglMakeCurrentFun;

procedure InitializeOpenGl;
begin
  OpenGl32DllHandle := LoadLibrary(opengl32);
  Glu32DllHandle := LoadLibrary('glu32.dll');

  if OpenGl32DllHandle <> 0 then
  begin
    @glGetStringFun := GetProcAddress(OpenGl32DllHandle, PChar('glGetString'));
    @glGetErrorFun := GetProcAddress(OpenGl32DllHandle, PChar('glGetError'));
    @wglCreateContextFun := GetProcAddress(OpenGl32DllHandle, PChar('wglCreateContext'));
    @wglDeleteContextFun := GetProcAddress(OpenGl32DllHandle, PChar('wglDeleteContext'));
    @wglMakeCurrentFun := GetProcAddress(OpenGl32DllHandle, PChar('wglMakeCurrent'));
  end
  else
  begin
    @glGetStringFun := nil;
    @glGetErrorFun := nil;
    @wglCreateContextFun := nil;
    @wglDeleteContextFun := nil;
    @wglMakeCurrentFun := nil;
  end;
  if Glu32DllHandle <> 0 then
    @gluErrorStringFun := GetProcAddress(Glu32DllHandle, PChar('gluErrorString'))
  else
    @gluErrorStringFun := nil;
end;

procedure CheckAndInitializeOpenGL;
begin
  if not OpenGLInitialized then
  begin
    InitializeOpenGl;
    OpenGLInitialized := True;
  end;
end;

function JclTryEnterCriticalSection(lpCriticalSection: TRTLCriticalSection): Boolean;
begin
  if Assigned(TryEnterCriticalSectionFun) then
    Result := TryEnterCriticalSectionFun(lpCriticalSection)
  else
    Result := False;
end;

function JclSignalObjectAndWait(hObjectToSignal: THandle; hObjectToWaitOn: THandle; dwMilliseconds: Cardinal; bAlertable: Boolean): Cardinal;
begin
  if Assigned(SignalObjectAndWaitFun) then
    Result := SignalObjectAndWaitFun(hObjectToSignal, hObjectToSignal, dwMilliseconds, bAlertable)
  else
    Result := WAIT_FAILED;
end;

function JclSetCriticalSectionSpinCount(lpCriticalSection: TRTLCriticalSection; dwSpinCount: Cardinal): Cardinal;
begin
  if Assigned(SetCriticalSectionSpinCountFun) then
    Result := SetCriticalSectionSpinCountFun(lpCriticalSection, dwSpinCount)
  else
    Result := 0;
end;

function JclOpenWaitableTimer(dwDesiredAccess: Cardinal; bInheritHandle: Boolean; const lpTimerName: string): THandle;
begin
  if Assigned(OpenWaitableTimerAFun) then
    Result := OpenWaitableTimerAFun(dwDesiredAccess, bInheritHandle, PChar(lpTimerName))
  else
    Result := 0;
end;

function JclInitializeCriticalSectionAndSpinCount(lpCriticalSection: TRTLCriticalSection; dwSpinCount: Cardinal): Boolean;
begin
  if Assigned(InitializeCriticalSectionAndSpinCountFun) then
    Result := InitializeCriticalSectionAndSpinCountFun(lpCriticalSection, dwSpinCount)
  else
    Result := False;
end;

function JclGetFileAttributesEx(const lpFileName: string; fInfoLevelId: TGetFileExInfoLevels; lpFileInformation: Pointer): Boolean;
begin
  if Assigned(GetFileAttributesExAFun) then
    Result := GetFileAttributesExAFun(PChar(lpFileName), fInfoLevelId, lpFileInformation)
  else
    Result := False;
end;

function JclCreateWaitableTimer(lpTimerAttributes: PSecurityAttributes; bManualReset: Boolean; const lpTimerName: string): THandle;
begin
  if Assigned(CreateWaitableTimerAFun) then
    Result := CreateWaitableTimerAFun(lpTimerAttributes, bManualReset, PChar(lpTimerAttributes))
  else
    Result := 0;
end;

function JclCancelWaitableTimer(hTimer: THandle): Boolean;
begin
  if Assigned(CancelWaitableTimerFun) then
    Result := CancelWaitableTimerFun(hTimer)
  else
    Result := False;
end;

function JclglGetString(name: Cardinal): PChar;
begin
  if Assigned(glGetStringFun) then
    Result := glGetStringFun(name)
  else
    Result := nil;
end;

function JclglGetError: Cardinal;
const
  GL_NO_ERROR = 0;
begin
  if Assigned(glGetErrorFun) then
    Result := glGetErrorFun
  else
    Result := GL_NO_ERROR;
end;

function JclwglCreateContext(hdc: HDC): HGLRC;
begin
  if Assigned(wglCreateContextFun) then
    Result := wglCreateContextFun(hdc)
  else
    Result := 0;
end;

function JclwglDeleteContext(hglrc: HGLRC): BOOL;
begin
  if Assigned(wglDeleteContextFun) then
    Result := wglDeleteContextFun(hglrc)
  else
    Result := False;
end;

function JclwglMakeCurrent(hdc: HDC; hglrc: HGLRC): BOOL;
begin
  if Assigned(wglMakeCurrentFun) then
    Result := wglMakeCurrentFun(hdc, hglrc)
  else
    Result := False;
end;

function JclgluErrorString(errCode: Cardinal): PChar;
begin
  if Assigned(gluErrorStringFun) then
    Result := gluErrorStringFun(errCode)
  else
    Result := nil;
end;


procedure InitializeKernel32;
begin
  Kernel32DllHandle := LoadLibrary(kernel32);
  if Kernel32DllHandle <> 0 then
  begin
    @TryEnterCriticalSectionFun := GetProcAddress(Kernel32DllHandle, PChar('TryEnterCriticalSection'));
    @SignalObjectAndWaitFun  := GetprocAddress(Kernel32DllHandle, PChar('SignalObjectAndWait'));
    @SetCriticalSectionSpinCountFun := GetProcAddress(Kernel32DllHandle, PChar('SetCriticalSectionSpinCount'));
    @OpenWaitableTimerAFun := GetProcAddress(Kernel32DllHandle, PChar('OpenWaitableTimerA'));
    @InitializeCriticalSectionAndSpinCountFun := GetProcAddress(Kernel32DllHandle, PChar('InitializeCriticalSectionAndSpinCount'));
    @GetFileAttributesExAFun := GetProcAddress(Kernel32DllHandle, PChar('GetFileAttributesExA'));
    @CreateWaitableTimerAFun := GetProcAddress(Kernel32DllHandle, PChar('CreateWaitableTimerA'));
    @CancelWaitableTimerFun := GetProcAddress(Kernel32DllHandle, PChar('CancelWaitableTimer'));
  end
  else
  begin
    @TryEnterCriticalSectionFun := nil;
    @SignalObjectAndWaitFun := nil;
    @SetCriticalSectionSpinCountFun := nil;
    @OpenWaitableTimerAFun := nil;
    @InitializeCriticalSectionAndSpinCountFun := nil;
    @GetFileAttributesExAFun := nil;
    @CreateWaitableTimerAFun := nil;
    @CancelWaitableTimerFun := nil;
  end;
end;


initialization
  InitializeKernel32;
  OpenGLInitialized := False;

finalization

  if Kernel32DllHandle <> 0 then
    FreeLibrary(Kernel32DllHandle);
  if OpenGl32DllHandle <> 0 then
    FreeLibrary(OpenGl32DllHandle);
  if Glu32DllHandle <> 0 then
    FreeLibrary(Glu32DllHandle);

end.