unit Mainfrm;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ActnList, StdCtrls, ComCtrls, JvUpDown, ExtCtrls,
  JvComponent, JvCalendar, Spin, JvDateTimePicker, Grids, TimeZones,
  Mask, JvExExtCtrls, JvClock, JvExComCtrls, JvExtComponent;

type
  TForm1 = class(TForm)
    PageControl1: TPageControl;
    DateAndTimeTab: TTabSheet;
    TimeZoneTab: TTabSheet;
    Button1: TButton;
    Button2: TButton;
    Button3: TButton;
    ActionList1: TActionList;
    OKAction: TAction;
    CancelAction: TAction;
    ApplyAction: TAction;
    Label1: TLabel;
    CurrentTimeZoneLabel: TLabel;
    GroupBox1: TGroupBox;
    GroupBox2: TGroupBox;
    JvUpDown1: TJvUpDown;
    YearSpin: TSpinEdit;
    MonthCombo: TComboBox;
    DayPicker: TStringGrid;
    TimeZonesCombo: TComboBox;
    Image1: TImage;
    AutoAdjustcb: TCheckBox;
    SetDaylightSavingsOptionsAction: TAction;
    TabSheet1: TTabSheet;
    CurrentTimeslv: TListView;
    GroupBox3: TGroupBox;
    Label2: TLabel;
    Label3: TLabel;
    Supported: TLabel;
    Period: TLabel;
    Label4: TLabel;
    GMTOffset: TLabel;
    TimeEdit2: TMaskEdit;
    JvClock1: TJvClock;
    TabSheet2: TTabSheet;
    Button5: TButton;
    TimeZonesCombo2: TComboBox;
    Label5: TLabel;
    GroupBox4: TGroupBox;
    Label12: TLabel;
    CheckMonthCombo: TComboBox;
    CheckYearSpin: TSpinEdit;
    CheckDayCombo: TComboBox;
    Label6: TLabel;
    Label7: TLabel;
    StdStartDate: TLabel;
    Period2: TLabel;
    Label10: TLabel;
    DlStartDate: TLabel;
    Label8: TLabel;
    DateIsInDaylightSavings: TLabel;
    procedure ApplyActionUpdate(Sender: TObject);
    procedure OKActionExecute(Sender: TObject);
    procedure CancelActionExecute(Sender: TObject);
    procedure DayPickerSelectCell(Sender: TObject; ACol, ARow: Integer;
      var CanSelect: Boolean);
    procedure FormCreate(Sender: TObject);
    procedure MonthYearComboChange(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure SetDaylightSavingsOptionsActionExecute(Sender: TObject);
    procedure ApplyActionExecute(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure CurrentTimeslvSelectItem(Sender: TObject; Item: TListItem;
      Selected: Boolean);
    procedure TimeZonesComboChange(Sender: TObject);
    procedure AutoAdjustcbClick(Sender: TObject);
    procedure JvClock1GetTime(Sender: TObject; var ATime: TDateTime);
    procedure Button5Click(Sender: TObject);
  private
    DateTimeModified: Boolean;
    TimeZoneModified: Boolean;
    AutoAdjustModified: Boolean;
    TimeZones: TTimeZones;
    procedure PopulateDayGrid(Year, Month, Day: Word);
    procedure SetupGrid;
  end;

var
  Form1: TForm1;

implementation

uses
  JclGraphUtils, JclDateTime;

{$R *.dfm}

procedure TForm1.ApplyActionUpdate(Sender: TObject);
begin
  (Sender as TAction).Enabled := DateTimeModified or TimeZoneModified or AutoAdjustModified;
end;

procedure TForm1.OKActionExecute(Sender: TObject);
begin
  ApplyAction.Execute;
  Close;
end;

procedure TForm1.CancelActionExecute(Sender: TObject);
begin
  Close;
end;

procedure TForm1.DayPickerSelectCell(Sender: TObject; ACol,
  ARow: Integer; var CanSelect: Boolean);
begin
  // Can't select the headers
  if ARow = 0 then
  begin
    CanSelect := False;
    Exit;
  end;

  // Can't select empty squares
  if (Sender as TStringGrid).Cells[ACol, ARow] = ' ' then
  begin
    CanSelect := False;
    Exit;
  end;
  DateTimeModified := True;
end;

procedure TForm1.SetupGrid;
const
  Days: array [0..6] of Char = ('M', 'T', 'W', 'T', 'F', 'S', 'S');
var
  I, J: Integer;
begin
  DayPicker.Col := 0;
  DayPicker.Row := 0;
   // Clear any existing dates
  for I := 0 to DayPicker.ColCount - 1 do
    for J := 0 to DayPicker.RowCount - 1 do
      DayPicker.Cells[I, J] := ' ';

   // Set the header
  for I := 0 to 6 do
    DayPicker.Cells[I, 0] := Days[I];
end;

procedure TForm1.PopulateDayGrid(Year, Month, Day: Word);
var
  X, Y, I: Integer;
  GridStart, FirstDay: Integer;
  Select: TGridRect;
begin
  if Day = 0 then
    Day := StrToIntDef(DayPicker.Cells[DayPicker.Selection.Left, DayPicker.Selection.Top], 0);

  SetupGrid;

  if (Year = 0) and (Month = 0) and (Day = 0) then
    DecodeDate(Now, Year, Month, Day);

  // Get the first of the month so we know what our offset is
  FirstDay := DayOfWeek(EncodeDate(Year, Month, 1));

  // DayOfWeek returns Sunday as 1, whereas Monday is 1 for us
  if FirstDay = 1 then
    GridStart := 7
  else
    GridStart := FirstDay - 1;

  X := GridStart - 1; // Our grid is 0 based
  Y := 1;
  for I := 1 to DaysInMonth(EncodeDate(Year, Month, 1)) do
  begin
    DayPicker.Cells[X, Y] := IntToStr(I);
    if I = Day then // Highlight current day's selection
    begin
      // Changing just one coordinate at a time still fires the OnSelectCell function - disable it
      Select.Top := Y;
      Select.Left := X;
      Select.Bottom := Y;
      Select.Right := X;
      DayPicker.Selection := Select;
    end;
    Inc(X);
    if X = 7 then
    begin
      X := 0;
      Inc(Y);
    end;
  end;
end;

procedure TForm1.FormCreate(Sender: TObject);
var
  Year, Month, Day: Word;
  I: Integer;
  ListItem: TListItem;
begin
  PageControl1.ActivePageIndex := 0;

  DecodeDate(Now, Year, Month, Day);
  MonthCombo.ItemIndex := Month - 1;
  YearSpin.Value := Year;

  PopulateDayGrid(Year, Month, Day);

  TimeZones := TTimeZones.Create;

  CurrentTimeslv.Items.BeginUpdate;
  try
    // Populate the combobox
    for I := 0 to TimeZones.Count - 1 do
    begin
      TimeZonesCombo.Items.Add(TimeZones[I].DisplayDescription);
      TimeZonesCombo2.Items.Add(TimeZones[I].DisplayDescription);
      
      Listitem := CurrentTimeslv.Items.Add;
      ListItem.Caption := TimeZones[I].DisplayDescription;
      ListItem.SubItems.Add(DateTimeToStr(TimeZones[I].CurrentDateTime));
    end;
  finally
    CurrentTimeslv.Items.EndUpdate;
  end;

  // Set the itemindex to the current timezone
  TimeZonesCombo.ItemIndex := TimeZonesCombo.Items.IndexOf(TimeZones.ActiveTimeZone.DisplayDescription);
  TimeZonesCombo2.ItemIndex := TimeZonesCombo2.Items.IndexOf(TimeZones.ActiveTimeZone.DisplayDescription);

  CurrentTimeslv.Selected := CurrentTimeslv.items[TimeZonesCombo.ItemIndex];
  CurrentTimeslv.Selected.MakeVisible(False);

  CurrentTimeZoneLabel.Caption := GetCurrentTimeZoneDescription;

  // Now set/unset the auto-adjust checkbox accordingly
  SetDaylightSavingsOptionsAction.Execute;

  AutoAdjustModified := False;
  DateTimeModified := False;
  TimeZoneModified := False;
end;

procedure TForm1.MonthYearComboChange(Sender: TObject);
begin
  PopulateDayGrid(YearSpin.Value, MonthCombo.ItemIndex + 1, 0);
  DateTimeModified := True;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  FreeAndNil(TimeZones);
end;

procedure TForm1.SetDaylightSavingsOptionsActionExecute(Sender: TObject);
begin
  AutoAdjustcb.Visible := TimeZones[TimeZonesCombo.ItemIndex].SupportsDaylightSavings;
  AutoAdjustcb.Checked := TimeZones.AutoAdjustEnabled;
end;

procedure TForm1.ApplyActionExecute(Sender: TObject);
var
  DateTime: TDateTime;
  Day: Integer;
begin
  // Set the timezone
  if TimeZoneModified then
  begin
    TimeZones[TimeZonesCombo.ItemIndex].ApplyTimeZone;
    CurrentTimeZoneLabel.Caption := GetCurrentTimeZoneDescription;
  end;

  // Set the date/time
  if DateTimeModified then
  begin
    Day := StrToInt(DayPicker.Cells[DayPicker.Selection.Left, DayPicker.Selection.Top]);
    DateTime := EncodeDate(YearSpin.Value, MonthCombo.ItemIndex + 1, Day);

    DateTime := DateTime + Frac(Now);
    TimeZones.SetDateTime(DateTime);
  end;

  if AutoAdjustModified then
    TimeZones.SetAutoAdjustEnabled(AutoAdjustcb.checked);

  DateTimeModified := False;
  TimeZoneModified := False;
  AutoAdjustModified := False;
end;

procedure TForm1.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  JvClock1.Enabled := False;
end;

procedure TForm1.CurrentTimeslvSelectItem(Sender: TObject; Item: TListItem;
  Selected: Boolean);
begin
  if (Item.Index <> -1) and Assigned(TimeZones) then
  begin
    GMTOffset.Caption := TimeZones[Item.Index].GMTOffset;
    if TimeZones[Item.Index].SupportsDaylightSavings then
      Supported.Caption := 'Yes'
    else
      Supported.Caption := 'No';

    Period.Caption := TimeZones[Item.Index].DayLightSavingsPeriod;
  end;
end;

procedure TForm1.TimeZonesComboChange(Sender: TObject);
begin
  // TODO: Make the change in the listview
  // TODO: Move the map
  TimeZoneModified := True;
  SetDaylightSavingsOptionsAction.Execute;
end;

procedure TForm1.AutoAdjustcbClick(Sender: TObject);
begin
  AutoAdjustModified := True;
end;

procedure TForm1.JvClock1GetTime(Sender: TObject; var ATime: TDateTime);
var
  I: Integer;
  Hour, Min, Sec, MSec: Word;
begin
  DecodeTime(ATime, Hour, Min, Sec, MSec);
  TimeEdit2.Text := Format('%.2d:%.2d:%.2d', [Hour, Min, Sec]);
  CurrentTimeslv.Items.BeginUpdate;
  try
    for I := 0 to CurrentTimeslv.Items.Count - 1 do
      CurrentTimeslv.Items[I].SubItems[0] := DateTimeToStr(TimeZones[I].CurrentDateTime);
  finally
    CurrentTimeslv.Items.EndUpdate;
  end;
end;

procedure TForm1.Button5Click(Sender: TObject);
var
  TZ: TTimeZoneInfo;
  Date: TDateTime;
  d,m: word;
  y: integer;
begin
  if (TimeZonesCombo2.ItemIndex = -1) or (CheckDayCombo.ItemIndex= -1) or (CheckMonthCombo.ItemIndex= -1) then
    Exit;

  y:=CheckYearSpin.Value;
  d:=CheckDayCombo.ItemIndex + 1;
  m:=CheckMonthCombo.ItemIndex + 1;

  try
    Date:=EncodeDate(y,m,d);
  except
    on E:EConvertError do
    begin
      Showmessage('Not a valid date');
      Exit;
    end;
  end;

  TZ:=TimeZones[TimeZonesCombo2.ItemIndex];
  if not TZ.SupportsDaylightSavings then
  begin
    Showmessage('This timezone does not support daylight savings');
    Exit;
  end;

  DLStartDate.Caption:=DateToStr(TZ.DaylightStartDateInYear(y));
  StdStartDate.Caption:=DateToStr(TZ.StandardStartDateInYear(y));

  Period2.Caption := TZ.DayLightSavingsPeriod;

  if TZ.DateTimeIsInDaylightSavings(Date) then
    DateIsInDaylightSavings.Caption:='Yes'
  else
    DateIsInDaylightSavings.Caption:='No';
end;

end.

