{ **************************************************************************** }
{                                                                              }
{    Pascal PreProcessor PChar utilities                                       }
{    Copyright (c) 2001 Barry Kelly.                                           }
{    barry_j_kelly@hotmail.com                                                 }
{                                                                              }
{    The contents of this file are subject to the Mozilla Public License       }
{    Version 1.1 (the "License"); you may not use this file except in          }
{    compliance with the License. You may obtain a copy of the License at      }
{    http://www.mozilla.org/MPL/                                               }
{                                                                              }
{    Software distributed under the License is distributed on an "AS IS"       }
{    basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the   }
{    License for the specific language governing rights and limitations        }
{    under the License.                                                        }
{                                                                              }
{    The Original Code is PCharUtils.pas                                       }
{                                                                              }
{    The Initial Developer of the Original Code is Barry Kelly.                }
{    Portions created by Barry Kelly are Copyright (C) 2001                    }
{    Barry Kelly. All Rights Reserved.                                         }
{                                                                              }
{    Alternatively, the contents of this file may be used under the terms      }
{    of the Lesser GNU Public License (the  "LGPL License"), in which case     }
{    the provisions of LGPL License are applicable instead of those            }
{    above.  If you wish to allow use of your version of this file only        }
{    under the terms of the LPGL License and not to allow others to use        }
{    your version of this file under the MPL, indicate your decision by        }
{    deleting  the provisions above and replace  them with the notice and      }
{    other provisions required by the LGPL License.  If you do not delete      }
{    the provisions above, a recipient may use your version of this file       }
{    under either the MPL or the LPGL License.                                 }
{                                                                              }
{ **************************************************************************** }
{ $Id: PCharUtils.pas 3094 2009-12-21 20:13:35Z outchy $ }

unit PCharUtils;

interface

uses SysUtils;

function SkipWhite(cp: PChar): PChar;
function ReadStringDoubleQuotedMaybe(cp: PChar; var AStr: string): PChar;
function ReadStringSingleQuotedMaybe(cp: PChar; var AStr: string): PChar;
function ReadIdent(cp: PChar; var ident: string): PChar;

implementation

function SkipWhite(cp: PChar): PChar;
begin
  while cp^ in [#1..#32] do
    Inc(cp);
  Result := cp;
end;

function ReadStringDoubleQuotedMaybe(cp: PChar; var AStr: string): PChar;
begin
  { possibly quoted string }
  Result := cp;
  if Result^ = '"' then
  begin
    while not (Result^ in [#0, '"']) do
      Inc(Result);
    if Result^ = #0 then
      raise Exception.Create('Unterminated string');
    Inc(Result); // skip over final "
    SetString(AStr, cp, Result - cp);
  end
  else
  begin
    while not (Result^ in [#0..#32]) do
      Inc(Result);
    SetString(AStr, cp, Result - cp);
  end;
end;

function ReadStringSingleQuotedMaybe(cp: PChar; var AStr: string): PChar;
begin
  { possibly quoted string }
  Result := cp;
  if Result^ = '"' then
  begin
    while not (Result^ in [#0, '"']) do
      Inc(Result);
    if Result^ = #0 then
      raise Exception.Create('Unterminated string');
    Inc(Result); // skip over final "
    SetString(AStr, cp, Result - cp);
  end
  else
  begin
    while not (Result^ in [#0..#32]) do
      Inc(Result);
    SetString(AStr, cp, Result - cp);
  end;
end;

function ReadIdent(cp: PChar; var ident: string): PChar;
var
  start: PChar;
begin
  start := cp;
  while cp^ in ['a'..'z', 'A'..'Z', '0'..'9', '_'] do
    Inc(cp);
  SetString(ident, start, cp - start);
  Result := cp;
end;

end.
