{

  TZipWriter: class optimized for large zip archive creation with as few IO
    operations as possible (i.e. no temporary files and compression/crc calc
    done in one pass)

  TZipReader: class for enumerating and decompressing files from a zip archive


  CRC32 Table and calculation taken from Indy (IdHashCRC.pas)


  IMPORTANT!!!
  =======================

  JCL 1.95 has a bug in it's TJclZLibCompressStream class that doesn't initialize the
  compression stream properly. this unit assumes that bug has been corrected.

  ZLibCheck(deflateInit(ZLibRecord, FCompressionLevel));

  should instead read:

  ZLibCheck(deflateInit2(FZLibStream, Level, Z_DEFLATED, WindowBits, DEF_MEM_LEVEL, Strategy));

 }


// TODO: encryption
// TODO: multidisk ZIP files
// TODO: file attributes


unit uZipHelper;

interface

uses
  Windows, SysUtils, Classes, Masks, ZLibH, JclCompression;

type
  //TZipPathFormat = (zpfNone, zpfRelative, zpfAbsolute);

  TZipEntry = record
    FileName: string;
    Position: integer;
    UncompressedSize: cardinal;
    CompressedSize: cardinal;
    CompressionMethod: cardinal;
    CRC32: cardinal;
    Modified: TDateTime;
    Comment: string;
    Attr: cardinal;
    Encrypted: boolean;
    // private vars that are used by find routines:
    FindMask: string;
    FindPos: cardinal;
    FindCount: cardinal;
  end;

  TZipWriter = class
  private
    FStream: TStream;
    FStreamOwned: boolean;
    FStreamSeek: boolean;
    FClosed: boolean;
    FList: TStringList;
    FUncompressedSize,
    FCompressedSize: cardinal;
    FCompressionLevel: TJclCompressionLevel;
    FFileCount: cardinal;
    FFailures: TStringList;
    procedure Clear;
  public
    property UncompressedSize: cardinal read FUncompressedSize;
    property CompressedSize: cardinal read FCompressedSize;
    property FileCount: cardinal read FFileCount;
    property Failures: TStringList read FFailures;
    property CompressionLevel: TJclCompressionLevel read FCompressionLevel write FCompressionLevel;
    // methods:
    constructor Create(const Stream: TStream; StreamSeekable: boolean = TRUE); overload; virtual;
    constructor Create(const FileName: string); overload; virtual;
    destructor Destroy; override;
    procedure Add(const FileName: string; const FileStream: TStream; Compress: boolean = TRUE); overload;
    procedure Add(const FileName: string; const RelativePath: string = ''; Compress: boolean = TRUE); overload;
    procedure BatchAdd(const Mask: string; const RelativePath: string = ''; Compress: boolean = TRUE);
    procedure Close;
  end;

  TZipReader = class
  private
    FStream: TStream;
    FStreamOwned: boolean;
    FComment: string;
  public
    constructor Create(const Stream: TStream); overload; virtual;
    constructor Create(const FileName: string); overload; virtual;
    destructor Destroy; override;
    function FindFirst(const Mask: string; var ZipEntry: TZipEntry): boolean;
    function FindNext(var ZipEntry: TZipEntry): boolean;
    procedure Extract(ZipEntry: TZipEntry; FileStream: TStream); overload; virtual;
    procedure Extract(ZipEntry: TZipEntry; const FileName: string); overload; virtual;
    procedure BatchExtract(const Mask: string; const RootPath: string; OverWriteExisting: boolean);
  end;



implementation

const
  {$REGION 'CRC32 Table'}
  CRC32Table : array[0..255] of longword = (
    $00000000, $77073096, $ee0e612c, $990951ba, $076dc419, $706af48f,
    $e963a535, $9e6495a3, $0edb8832, $79dcb8a4, $e0d5e91e, $97d2d988,
    $09b64c2b, $7eb17cbd, $e7b82d07, $90bf1d91, $1db71064, $6ab020f2,
    $f3b97148, $84be41de, $1adad47d, $6ddde4eb, $f4d4b551, $83d385c7,
    $136c9856, $646ba8c0, $fd62f97a, $8a65c9ec, $14015c4f, $63066cd9,
    $fa0f3d63, $8d080df5, $3b6e20c8, $4c69105e, $d56041e4, $a2677172,
    $3c03e4d1, $4b04d447, $d20d85fd, $a50ab56b, $35b5a8fa, $42b2986c,
    $dbbbc9d6, $acbcf940, $32d86ce3, $45df5c75, $dcd60dcf, $abd13d59,
    $26d930ac, $51de003a, $c8d75180, $bfd06116, $21b4f4b5, $56b3c423,
    $cfba9599, $b8bda50f, $2802b89e, $5f058808, $c60cd9b2, $b10be924,
    $2f6f7c87, $58684c11, $c1611dab, $b6662d3d, $76dc4190, $01db7106,
    $98d220bc, $efd5102a, $71b18589, $06b6b51f, $9fbfe4a5, $e8b8d433,
    $7807c9a2, $0f00f934, $9609a88e, $e10e9818, $7f6a0dbb, $086d3d2d,
    $91646c97, $e6635c01, $6b6b51f4, $1c6c6162, $856530d8, $f262004e,
    $6c0695ed, $1b01a57b, $8208f4c1, $f50fc457, $65b0d9c6, $12b7e950,
    $8bbeb8ea, $fcb9887c, $62dd1ddf, $15da2d49, $8cd37cf3, $fbd44c65,
    $4db26158, $3ab551ce, $a3bc0074, $d4bb30e2, $4adfa541, $3dd895d7,
    $a4d1c46d, $d3d6f4fb, $4369e96a, $346ed9fc, $ad678846, $da60b8d0,
    $44042d73, $33031de5, $aa0a4c5f, $dd0d7cc9, $5005713c, $270241aa,
    $be0b1010, $c90c2086, $5768b525, $206f85b3, $b966d409, $ce61e49f,
    $5edef90e, $29d9c998, $b0d09822, $c7d7a8b4, $59b33d17, $2eb40d81,
    $b7bd5c3b, $c0ba6cad, $edb88320, $9abfb3b6, $03b6e20c, $74b1d29a,
    $ead54739, $9dd277af, $04db2615, $73dc1683, $e3630b12, $94643b84,
    $0d6d6a3e, $7a6a5aa8, $e40ecf0b, $9309ff9d, $0a00ae27, $7d079eb1,
    $f00f9344, $8708a3d2, $1e01f268, $6906c2fe, $f762575d, $806567cb,
    $196c3671, $6e6b06e7, $fed41b76, $89d32be0, $10da7a5a, $67dd4acc,
    $f9b9df6f, $8ebeeff9, $17b7be43, $60b08ed5, $d6d6a3e8, $a1d1937e,
    $38d8c2c4, $4fdff252, $d1bb67f1, $a6bc5767, $3fb506dd, $48b2364b,
    $d80d2bda, $af0a1b4c, $36034af6, $41047a60, $df60efc3, $a867df55,
    $316e8eef, $4669be79, $cb61b38c, $bc66831a, $256fd2a0, $5268e236,
    $cc0c7795, $bb0b4703, $220216b9, $5505262f, $c5ba3bbe, $b2bd0b28,
    $2bb45a92, $5cb36a04, $c2d7ffa7, $b5d0cf31, $2cd99e8b, $5bdeae1d,
    $9b64c2b0, $ec63f226, $756aa39c, $026d930a, $9c0906a9, $eb0e363f,
    $72076785, $05005713, $95bf4a82, $e2b87a14, $7bb12bae, $0cb61b38,
    $92d28e9b, $e5d5be0d, $7cdcefb7, $0bdbdf21, $86d3d2d4, $f1d4e242,
    $68ddb3f8, $1fda836e, $81be16cd, $f6b9265b, $6fb077e1, $18b74777,
    $88085ae6, $ff0f6a70, $66063bca, $11010b5c, $8f659eff, $f862ae69,
    $616bffd3, $166ccf45, $a00ae278, $d70dd2ee, $4e048354, $3903b3c2,
    $a7672661, $d06016f7, $4969474d, $3e6e77db, $aed16a4a, $d9d65adc,
    $40df0b66, $37d83bf0, $a9bcae53, $debb9ec5, $47b2cf7f, $30b5ffe9,
    $bdbdf21c, $cabac28a, $53b39330, $24b4a3a6, $bad03605, $cdd70693,
    $54de5729, $23d967bf, $b3667a2e, $c4614ab8, $5d681b02, $2a6f2b94,
    $b40bbe37, $c30c8ea1, $5a05df1b, $2d02ef8d);
  {$ENDREGION}

  ZIP_FILE_SIG = $04034B50;  
  ZIP_DIR_HEADER_SIG = $02014B50;
  ZIP_DIR_FOOTER_SIG = $06054B50;

  ZIP_METHOD_STORED = 0;
  ZIP_METHOD_DEFLATE = 8;

  ZIP_FLAG_ENCRYPTED = 1;
  ZIP_FLAG_NOSEEK = 8;

type
  {$REGION 'Internal ZIP Structures'}

  {
    Overall zipfile format:

    [local file header + file data + data_descriptor] . . .
    [central directory] end of central directory record
  }

  PLocalFileHeader = ^TLocalFileHeader;
  TLocalFileHeader = packed record
    Sig: longword;
    Version: word;
    Flags: word;
    Method: word;
    ModTime: word;
    ModDate: word;
    CRC32: longword;
    CompressedSize: longword;
    UncompressedSize: longword;
    FilenameLength: word;
    ExtraLength: word;
    // filename (variable size)
    // extra field (variable size)
  end;

  {
    This descriptor exists only if bit 3 of the general
    purpose bit flag is set (see below).  It is byte aligned
    and immediately follows the last byte of compressed data.
    This descriptor is used only when it was not possible to
    seek in the output zip file, e.g., when the output zip file
    was standard output or a non seekable device.
  }
  TLocalDataDescriptor = packed record
    CRC32: longword;
    CompressedSize: longword;
    UncompressedSize: longword;
  end;

  PCentralDirHeader = ^TCentralDirHeader;
  TCentralDirHeader = packed record
    Sig: longword;
    OrigVersion, ReadVersion: word;
    Flags: word;
    Method: word;
    ModTime: word;
    ModDate: word;
    CRC32: longword;
    CompressedSize: longword;
    UncompressedSize: longword;
    FileNameLength: word;
    ExtraLength: word;
    CommentLenth: word;
    FirstDisk: word;
    IntAttrib: word;
    ExtAttrib: longword;
    Offset: longword;
    // file name (variable size)
    // extra field (variable size)
    // file comment (variable size)
  end;

  TCentralDirFooter = packed record
    Sig: longword;
    ThisDisk: word;
    FirstDirDisk: word;
    DirEntriesThisDisk: word;
    DirEntries: word;
    DirSize: longword;
    DirOffset: longword;
    CommentLength: word;
    // zipfile comment (variable size)
  end;
  {$ENDREGION}


function CopyStreamWithCRC32(Source, Dest: TStream; Size: integer): LongWord;
// purpose: copy a stream and calculate the CRC32 checksum in a single pass
var
  LByte: ^byte;
  Buf: array[1..$F000] of byte;
  Count, I: integer;
begin
  Result := $FFFFFFFF;

  repeat
    Count := SizeOf(Buf);
    if Count > Size then
      Count := Size;

    Source.ReadBuffer(Buf, Count);
    Dest.WriteBuffer(Buf, Count);

    // calculate source crc
    LByte := @Buf;
    for I := 1 to Count do begin
      Result := ((Result shr 8) and $00FFFFFF) xor CRC32Table[(Result xor LByte^) and $FF];
      Inc(LByte);
    end;

    Dec(Size, Count);
  until Size = 0;

  Result := Result xor $FFFFFFFF;
end;  


{ TZipWriter }

constructor TZipWriter.Create(const Stream: TStream; StreamSeekable: boolean);
// note: StreamSeekable indicates if the Stream supports random access. Setting
//  this to FALSE is highly discouraged because it means the program that reads
//  this zip file will need to decompress it all just to build a list of files.
begin
  inherited Create;
  FStream := Stream;
  FStreamOwned := FALSE;
  FStreamSeek := StreamSeekable;
  FStream.Position := 0;
  FClosed := FALSE;
  FList := TStringList.Create;
  FUncompressedSize := 0;
  FCompressedSize := 0;
  FCompressionLevel := Z_DEFAULT_COMPRESSION;
  FFailures := TStringList.Create;
  FFileCount := 0;
end;

constructor TZipWriter.Create(const FileName: string);
begin
  Create(TFileStream.Create(FileName, fmCreate or fmShareDenyWrite), TRUE);
  FStreamOwned := TRUE;
end;

destructor TZipWriter.Destroy;
begin
  Close;
  if FStreamOwned then
    FStream.Free;
  Clear;
  FList.Free;
  FFailures.Free;
  inherited;
end;

procedure TZipWriter.Clear;
// purpose: clear central directoy
var
  I: integer;
  Dir: PCentralDirHeader;
begin
  for I := 0 to Pred(FList.Count) do begin
    Dir := Pointer(FList.Objects[I]);
    Dispose(Dir);
  end;
  FList.Clear;
end;

procedure TZipWriter.Close;
// purpose: write central directory to zip stream. this contains a list of all the
//  files written so far
var
  FileName: string;
  DirPos: cardinal;
  Dir: PCentralDirHeader;
  DirFoot: TCentralDirFooter;
  I: integer;
begin
  if FClosed then
    Exit;

  DirPos := FStream.Position;
  for I := 0 to Pred(FList.Count) do begin
    FileName := FList[I];
    Dir := Pointer(FList.Objects[I]);
    FStream.WriteBuffer(Dir^, SizeOf(Dir^));
    if Length(FileName) > 0 then
      FStream.WriteBuffer(FileName[1], Length(FileName));
  end;

  FillChar(DirFoot, Sizeof(DirFoot), #0);
  DirFoot.Sig := ZIP_DIR_FOOTER_SIG;
  DirFoot.DirEntriesThisDisk := FList.Count;
  DirFoot.DirEntries := FList.Count;
  DirFoot.DirSize := FStream.Position - DirPos;
  DirFoot.DirOffset := DirPos;
  FStream.WriteBuffer(DirFoot, SizeOf(DirFoot));

  FClosed := TRUE;
end;

procedure TZipWriter.Add(const FileName: string; const FileStream: TStream; Compress: boolean);
// purpose: add a stream to the zip archive
var
  Head: TLocalFileHeader;
  Dir: PCentralDirHeader;
  Desc: TLocalDataDescriptor;
  DataBegin, DataEnd: integer;
  FileDate, FileTime: word;
  FileDateTime, LocalFileDateTime: TFileTime;
  SysTime: TSystemTime;
  ZLib: TJclZLibCompressStream;
begin
  if FClosed then
    raise Exception.Create('Zip file is closed. Cannot add more files.');

  if FileStream is TFileStream then begin
    GetFileTime(TFileStream(FileStream).Handle, nil, nil, @FileDateTime);
    FileTimeToLocalFileTime(FileDateTime, LocalFileDateTime);
    FileTimeToDosDateTime(LocalFileDateTime, FileDate, FileTime);
  end
  else begin
    // note: filexists() will return false on files that exist but have no date.
    // although this doesn't affect us here, we will try to prevent a problem
    // downstream in another program by always storing a datetime
    //FileDate := 0;
    //FileTime := 0;
    DateTimeToSystemTime(Now, SysTime);
    SystemTimeToFileTime(SysTime, LocalFileDateTime);
    FileTimeToDosDateTime(LocalFileDateTime, FileDate, FileTime);
  end;

  // prepare headers
  FillChar(Head, SizeOf(Head), #0);
  Head.Sig := ZIP_FILE_SIG;
  Head.Version := 0; // 0 - MS-DOS and OS/2 (FAT / VFAT / FAT32 file systems)
  if Compress then
    Head.Method := ZIP_METHOD_DEFLATE
  else
    Head.Method := ZIP_METHOD_STORED;
  if not FStreamSeek then
    Head.Flags := Head.Flags or ZIP_FLAG_NOSEEK;
  Head.ModDate := FileDate;
  Head.ModTime := FileTime;
  Head.FilenameLength := Length(FileName);

  New(Dir);
  FList.AddObject(FileName, TObject(Dir));
  FillChar(Dir^, SizeOf(Dir^), #0);
  Dir^.Sig := ZIP_DIR_HEADER_SIG;
  Dir^.OrigVersion := Head.Version;
  Dir^.ReadVersion := Head.Version;
  Dir^.Flags := Head.Flags;
  Dir^.Method := Head.Method;
  Dir^.ModDate := Head.ModDate;
  Dir^.ModTime := Head.ModTime;
  Dir^.UncompressedSize := FileStream.Size;
  Dir^.FilenameLength := Head.FilenameLength;
  Dir^.Offset := FStream.Position;

  // write header
  FStream.WriteBuffer(Head, SizeOf(Head));
  if Length(FileName) > 0 then
    FStream.WriteBuffer(FileName[1], Length(FileName));

  // write file
  DataBegin := FStream.Position;
  if not Compress then begin
    //FStream.CopyFrom(FileStream, 0)
    Dir^.CRC32 := CopyStreamWithCRC32(FileStream, FStream, FileStream.Size);
  end
  else begin
    ZLib := TJclZLibCompressStream.Create(FStream, FCompressionLevel);
    ZLib.WindowBits := -15;
    Dir^.CRC32 := CopyStreamWithCRC32(FileStream, Zlib, FileStream.Size);
    ZLib.Free;
  end;
  DataEnd := FStream.Position;
  Dir^.CompressedSize := DataEnd - DataBegin;

  // backup a bit to fixup the header with new compressed length and CRC
  if FStreamSeek then begin
    Head.CRC32 := Dir^.CRC32;
    Head.UncompressedSize := Dir^.UnCompressedSize;
    Head.CompressedSize := Dir^.CompressedSize;
    FStream.Position := Dir^.Offset;
    FStream.WriteBuffer(Head, SizeOf(Head));
    FStream.Position := DataEnd;
  end
  // write data descriptor if seek not supported
  else begin
    Desc.CRC32 := Dir^.CRC32;
    Desc.UncompressedSize := Dir^.UncompressedSize;
    Desc.CompressedSize := Dir^.CompressedSize;
    FStream.WriteBuffer(Desc, SizeOf(Desc));
  end;

  Inc(FUncompressedSize, Dir^.UncompressedSize);
  Inc(FCompressedSize, Dir^.CompressedSize);
  Inc(FFileCount);
end;

procedure TZipWriter.Add(const FileName: string; const RelativePath: string; Compress: boolean);
// purpose: add a file to the zip archive
// note: there are several ways to save path names to an archive
//   1. with no paths: pass '' as the RelativePath
//   2. with relative paths: supply a RelativePath. all files will then be
//      stored with paths relative to that path.
//   3. with a full path (include drive): pass ':' as the RelativePath
//   4. with a full path (excluding drive): pass '\' as the RelativePath

var
  Path: string;
  Name: string;
  F: TFileStream;
  I: integer;
begin
  Name := FileName;

  if (RelativePath = ':') then
    Name := FileName
  else if (RelativePath = '\') then begin
    Name := FileName;
    if Copy(Name, 2, 2) = ':\' then
      Delete(Name, 1, 3);
  end
  else if (RelativePath <> '') then begin
    Path := IncludeTrailingBackslash(ExtractFilePath(Name));

    // if the filename is locaed in the rootpath, then strip away the rootpath
    if SameText(Copy(Path, 1, Length(RelativePath)), RelativePath) then
      Delete(Name, 1, Length(RelativePath));

    // strip any leading backslashes
    if Copy(Name, 1, 1) = '\' then
      Delete(Name, 1, 1);
  end
  else
    Name := ExtractFileName(Name);

  // switch to unix style paths
  Name := StringReplace(Name, '\', '/', [rfReplaceAll]);
  try
    F := TFileStream.Create(FileName, fmOpenRead or fmShareDenyWrite);
    try
      Add(Name, F, Compress);
    finally
      F.Free;
    end;

  except
    on E: Exception do begin
      FFailures.Add(FileName + ' (' + E.Message + ')');
    end;
  end;
end;

procedure TZipWriter.BatchAdd(const Mask, RelativePath: string; Compress: boolean);
var
  Path: string;
  SR: TSearchRec;
  I: integer;
begin
  Path := ExtractFilePath(Mask);
  I := FindFirst(Mask, faAnyFile, SR);
  if I = 0 then begin
    repeat
      if (SR.Attr and faDirectory) = 0 then
        Add(Path + SR.Name, RelativePath, Compress);
    until FindNext(SR) <> 0;
    FindClose(SR);
  end;
end;


{ TZipReader }

constructor TZipReader.Create(const Stream: TStream);
begin
 inherited Create;
 FStream := Stream;
 FStreamOwned := FALSE;
end;

constructor TZipReader.Create(const FileName: string);
begin
  Create(TFileStream.Create(FileName, fmOpenRead or fmShareDenyWrite));
  FStreamOwned := TRUE;
end;

destructor TZipReader.Destroy;
begin
  if FStreamOwned then
    FStream.Free;
  inherited;
end;

function TZipReader.FindFirst(const Mask: string; var ZipEntry: TZipEntry): boolean;
begin
  ZipEntry.FindPos := 0;
  ZipEntry.FindCount := 0;
  ZipEntry.FindMask := Mask;
  Result := FindNext(ZipEntry);
end;

function TZipReader.FindNext(var ZipEntry: TZipEntry): boolean;
var
  Head: TLocalFileHeader;
  Desc: TLocalDataDescriptor;
  DescSize: integer;
  I: integer;
  FileDateTime, LocalFileDateTime: TFileTime;
  SysTime: TSystemTime;
begin
  Result := FALSE;
  repeat
    // load next local file header
    FStream.Position := ZipEntry.FindPos;
    I := FStream.Read(Head, SizeOf(Head));

    // exit if we hit the end of the archive or the signature is invalid
    if (I < SizeOf(Head)) or (Head.Sig <> ZIP_FILE_SIG) then
      Break;

    // todo: implement non-seekable zip reading
    if (Head.Flags and ZIP_FLAG_NOSEEK) <> 0 then
      raise Exception.Create('Non-seekable ZIP files not supported');

    // read in variable length filename. it is acceptable to have a blank filename
    // (possibly when the input was compressed directly from the stdin)
    if (Head.FilenameLength > 0) then begin
      SetLength(ZipEntry.FileName, Head.FilenameLength);
      I := FStream.Read(ZipEntry.FileName[1], Head.FilenameLength);
      if (I < Head.FilenameLength) then
        Break;
    end
    else
      ZipEntry.FileName := '__MissingFileName' + IntToStr(ZipEntry.FindPos) + '__';

    // switch to windows style path delimitters
    ZipEntry.FileName := StringReplace(ZipEntry.FileName, '/', '\', [rfReplaceAll]);

    // does this file contain a fixup data descriptor?
    if (Head.Flags and ZIP_FLAG_NOSEEK) <> 0 then
      DescSize := SizeOf(Desc)
    else
      DescSize := 0;

    // set remaining properties if the filename matches the search mask
    if MatchesMask(ZipEntry.FileName, ZipEntry.FindMask) then begin
      ZipEntry.Position := ZipEntry.FindPos + SizeOf(Head) + Head.FilenameLength + Head.ExtraLength;
      ZipEntry.Encrypted := (Head.Flags and ZIP_FLAG_ENCRYPTED) <> 0;
      if DescSize = 0 then begin
        ZipEntry.CRC32 := Head.CRC32;
        ZipEntry.CompressedSize := Head.CompressedSize;
        ZipEntry.UncompressedSize := Head.UncompressedSize;
      end
      else begin
        FStream.Position := ZipEntry.Position + Head.CompressedSize;
        FStream.Read(Desc, SizeOf(Desc));
        ZipEntry.CRC32 := Desc.CRC32;
        ZipEntry.CompressedSize := Desc.CompressedSize;
        ZipEntry.UncompressedSize := Desc.UncompressedSize;
      end;
      ZipEntry.CompressionMethod := Head.Method;
      DosDateTimeToFileTime(Head.ModDate, Head.ModTime, LocalFileDateTime);
      FileTimeToSystemTime(LocalFileDateTime, SysTime);
      ZipEntry.Modified := SystemTimeToDateTime(SysTime);
      if IsDelimiter('\', ZipEntry.FileName, Length(ZipEntry.FileName)) then
        ZipEntry.Attr := faDirectory
      else
        ZipEntry.Attr := 0;
      Inc(ZipEntry.FindCount);
      Result := TRUE;
    end;

    // move to the next file entry
    Inc(ZipEntry.FindPos, SizeOf(Head) + Head.FilenameLength + Head.ExtraLength + Head.CompressedSize + DescSize);

  until Result;
end;

procedure TZipReader.Extract(ZipEntry: TZipEntry; FileStream: TStream);
var
  ZLib: TJclZLibDecompressStream;
  FileDateTime, LocalFileDateTime: TFileTime;
  SysTime: TSystemTime;
  CRC32: longword;
begin
  if (ZipEntry.Attr and faDirectory) <> 0 then
    Exit;

  if ZipEntry.Encrypted then
    raise Exception.CreateFmt('%s has been encrypted using an unsupported method', [ZipEntry.FileName]);

  // decompress the file
  FStream.Position := ZipEntry.Position;
  if ZipEntry.CompressionMethod = ZIP_METHOD_STORED then
    FileStream.CopyFrom(FStream, ZipEntry.UncompressedSize)
  else if ZipEntry.CompressionMethod = ZIP_METHOD_DEFLATE then begin
    ZLib := TJclZLibDecompressStream.Create(FStream, -15);
    try
      CRC32 := CopyStreamWithCRC32(ZLib, FileStream, ZipEntry.UncompressedSize);
      if CRC32 <> ZipEntry.CRC32 then
        raise Exception.CreateFmt('%s has been corrupted', [ZipEntry.FileName]);
    finally
      ZLib.Free;
    end;
  end
  else
    raise Exception.CreateFmt('%s has been compressed using an unsupported method (%d)', [ZipEntry.FileName, ZipEntry.CompressionMethod]);

  // set file date to date from archive
  if FileStream is TFileStream then begin
    DateTimeToSystemTime(ZipEntry.Modified, SysTime);
    SystemTimeToFileTime(SysTime, LocalFileDateTime);
    LocalFileTimeToFileTime(LocalFileDateTime, FileDateTime);
    SetFileTime(TFileStream(FileStream).Handle, @FileDateTime, nil, @FileDateTime);
  end;
end;

procedure TZipReader.Extract(ZipEntry: TZipEntry; const FileName: string);
var
  F: TFileStream;
begin
  ForceDirectories(ExtractFilePath(FileName));
  F := TFileStream.Create(Filename, fmCreate);
  try
    Extract(ZipEntry, F);
  finally
    F.Free;
  end;
end;

procedure TZipReader.BatchExtract(const Mask, RootPath: string; OverWriteExisting: boolean);
var
  FindRec: TZipEntry;
  Path: string;
  F: TFileStream;
begin
  if FindFirst(Mask, FindRec) then
    repeat
      // some entries may be directories - ignore these
      if (FindRec.Attr and faDirectory) = 0 then begin
        Path := IncludeTrailingBackslash(RootPath) + FindRec.FileName;
        if (not FileExists(Path)) or OverWriteExisting then begin
          Extract(FindRec, Path);
        end;
      end;
    until not FindNext(FindRec);
end;



end.
